unit ANICursorList; { Animated cursor list component
                      c. LeslieSoft 1997-1999, lsoft@ax.hu }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, DsgnIntf, ANICursorListDlg;

type
  { Animated cursor object }
  TANICursor  = class(TObject)
                private
                  {Internal data fields }
                  FMemStream  : TMemoryStream;  { Stream to store animated cursor raw data }
                  FIcons      : TList;          { List of separated icons from the animated cursor }
                  FSteps      : LongInt;        { Steps in the animation }
                  FTitle      : String;         { Title string }
                  FAuthor     : String;         { Author string }
                  FJifRate    : LongInt;        { Default rate }
                  FRates      : TList;          { Rate table }
                  FSeq        : TList;          { Sequence }
                  FAnimation  : Boolean;        { True : animation in progress }
                  FTimer      : TTimer;         { Internal timer for animation }
                  FControl    : TWinControl;    { Control that shows the animation }
                  FX,                           { X position of the animation }
                  FY          : Integer;        { Y position of the animation }
                  FWidth,                       { Icon width for the animation }
                  FHeight     : Integer;        { Icon height for the animation }
                  FBrush      : TBrush;         { Background brush for the animation }
                  FCurStep    : Integer;        { Current animation step while animating }
                  { Property related methods }
                  function  GetTitle : String;
                  function  GetAuthor : String;
                  function  GetIcon(AStep : Integer) : TIcon;
                  function  GetStepCount : Integer;
                  procedure SetAnimation(AAnimation : Boolean);
                  { Other methods }
                  procedure SetFields;
                  procedure DrawCurIcon;
                protected
                  { Protected method }
                  procedure TimerEvent(Sender: TObject);
                public
                  { Public methods }
                  constructor Create;
                  destructor  Destroy; override;
                  procedure   Animate(Control : TWinControl; X,Y : Integer; Width,Height : Integer; Brush : TBrush; Start : Boolean);
                  procedure   DrawIcon(Canvas : TCanvas; X,Y : Integer; Step : Integer; Width,Height : Integer; Brush : TBrush);
                  procedure   ShowAnimation;
                  procedure   HideAnimation;
                  { Public properties }
                  property    Title : String read GetTitle;
                  property    Author : String read GetAuthor;
                  property    Icon[Step : Integer] : TIcon read GetIcon;
                  property    StepCount : Integer read GetStepCount;
                  property    Animation : Boolean read FAnimation write SetAnimation;
                end;

  { List of animated cursors object }
  TANICursorList  = class(TComponent)
                    private
                      { Internal data fields }
                      FCursors  : TList;        { List of TANICursor objects }
                      FOnChange : TNotifyEvent; { Called after every change of the list }
                      FModified : Boolean;      { True: modified since BeginUpdate last called }
                      FInUpdate : Boolean;      { True: BeginUpdate is called }
                      { property related methods }
                      function    GetTitle(Index : Integer) : String;
                      function    GetAuthor(Index : Integer) : String;
                      function    GetCount : Integer;
                      function    GetStepCount(Index: Integer) : Integer;
                      function    GetIcon(Index,Step : Integer) : TIcon;
                      function    GetAnimation(Index : Integer) : Boolean;
                      procedure   SetAnimation(Index : Integer; AAnimation : Boolean);
                    protected
                      { Protected methods }
                      procedure DefineProperties(Filer: TFiler); override;
                      procedure ReadData(Reader: TReader);
                      procedure WriteData(Writer: TWriter);
                      procedure Loaded; override;
                    public
                      { Public methods }
                      constructor Create(AOwner: TComponent); override;
                      destructor  Destroy; override;
                      function    Add(const FileName: TFileName): Integer;
                      procedure   Append(const FileName: TFileName);
                      procedure   Clear;
                      procedure   Delete(Index: Integer);
                      procedure   Exchange(Index1, Index2: Integer);
                      function    Find(const Title: string; var Index: Integer): Boolean;
                      function    IndexOf(const Title: string): Integer;
                      procedure   Insert(Index: Integer; const FileName: TFileName);
                      procedure   Move(CurIndex, NewIndex: Integer);
                      procedure   RegisterCursors;
                      procedure   SetCursor(Index: Integer);
                      procedure   Animate(Index : Integer; Control : TWinControl; X,Y : Integer; Width,Height : Integer; Brush : TBrush; Start : Boolean);
                      procedure   DrawIcon(Index,Step : Integer; Canvas : TCanvas; X,Y : Integer; Width,Height : Integer; Brush : TBrush);
                      procedure   ShowAnimation(Index : Integer);
                      procedure   HideAnimation(Index : Integer);
                      procedure   ShowAnimations;
                      procedure   HideAnimations;
                      procedure   BeginUpdate;
                      procedure   EndUpdate;
                      { Public properties }
                      property    Animation[Index : Integer] : Boolean read GetAnimation write SetAnimation;
                      property    Title[Index : Integer]: String read GetTitle;
                      property    Author[Index : Integer]: String read GetAuthor;
                      property    Icon[Index,Step : Integer]: TIcon read GetIcon;
                      property    StepCount[Index : Integer] : Integer read GetStepCount;
                      property    Count : Integer read GetCount;
                    published
                      { Published method }
                      property    OnChange: TNotifyEvent read FOnChange write FOnChange;
                    end;

  { Component editor for TANICursorList }
  TANICursorListEditor = class(TComponentEditor)
                         public
                           procedure Edit; override;
                           procedure ExecuteVerb(Index: Integer); override;
                           function GetVerb(Index: Integer): string; override;
                           function GetVerbCount: Integer; override;
                         end;

procedure Register;

implementation

type
  { ANI header record }
  TANIHeader = record
                 SizeOf   : LongInt; { Size of the record (=36) }
                 Frames   : LongInt; { Number of frames }
                 Steps    : LongInt; { Number of animations }
                 CX,CY    : LongInt; { 0,0 }
                 BitCount : LongInt; { 0 }
                 Planes   : LongInt; { 0 }
                 JifRate  : LongInt; { Default jiffies if no RATE chunk present }
                 Flags    : LongInt; { 0 }
               end;

const
  idRIFF = $46464952; { RIFF chunk identifier }
  idACON = $4E4F4341; { ACON chunk identifier }
  idLIST = $5453494C; { LIST chunk identifier }
  idINAM = $4D414E49; { INAM chunk identifier (= Title) }
  idIART = $54524149; { IART chunk identifier (= Author) }
  idFRAM = $6D617266; { FRAM chunk identifier }
  idICON = $6E6F6369; { ICON chunk identifier (= array of icons ) }
  idANIH = $68696E61; { ANIH chunk identifier (= ANIHeader) }
  idRATE = $65746172; { RATE chunk identifier (= delay after a step in jiffies ) }
  idSEQ  = $20716573; { SEQ  chunk identifier (= sequence of icons ) }
  idINFO = $4F464E49; { INFO chunk identifier }
  afICON = 1;         { Icon id }

  Speed  = 14;        { Constant to calculate animation speed }

{--- TANICursor methods ---}

{ Returns the title of the animated cursor }
function TANICursor.GetTitle : String;
begin
  Result := FTitle;
end;

{ Returns the author of the animated cursor }
function TANICursor.GetAuthor : String;
begin
  Result := FAuthor;
end;

{ Returns the AStepth icon from the animated cursor }
function TANICursor.GetIcon(AStep : Integer) : TIcon;
begin
  Result := TIcon(FIcons[LongInt(FSeq[AStep])]);
end;

{ Returns the number of steps of the animated cursor }
function TANICursor.GetStepCount : Integer;
begin
  Result := FSteps;
end;

{ Called by setting the Animation property }
procedure TANICursor.SetAnimation(AAnimation : Boolean);
begin
  if (FAnimation <> AAnimation) and Assigned(FControl) then
    begin
      FAnimation := AAnimation;
      if FAnimation
      then
        { Draw first step's icon and start the timer }
        begin
          FCurStep := 0;
          DrawCurIcon;
          FTimer.Interval := Speed * LongInt(FRates[FCurStep]);
          FTimer.Enabled := True;
        end
      else
        { Disable the timer and clear drawing area }
        begin
          FTimer.Enabled := False;
          FControl.Invalidate;
        end;
    end;
end;

{ Sets internal field from the memory image of the animated cursor }
procedure TANICursor.SetFields;
var
  ANIHeader  : TANIHeader;
  Ident      : LongInt;
  ChunkLen   : LongInt;
  Readed     : LongInt;
  FileLen    : LongInt;
  Buffer     : PChar;
  L          : LongInt;
  B          : Byte;
  I          : Integer;
  CMemStream : TMemoryStream;
  Icon       : TIcon;
const
  ByteOne : Byte = 1;
begin
  try
    FMemStream.Seek(0,soFromBeginning);
    { Read RIFF header - "RIFF" followed by the length of file }
    Readed := FMemStream.Read(Ident,4);
    if (Readed <> 4) or (Ident <> idRIFF) then
      Raise Exception.Create('');
    Readed := FMemStream.Read(FileLen,4);
    if Readed <> 4 then
      Raise Exception.Create('');
    { Read chunks }
    while FMemStream.Position < FMemStream.Size-1 do
      begin
        Readed := FMemStream.Read(Ident,4);
        if Readed <> 4 then
          Raise Exception.Create('');
        case Ident of
          { ACON chunk - do nothing }
          idACON : ;
          { FRAM chunk - do nothing }
          idFRAM : ;
          { INFO chunk - do nothing }
          idINFO : ;
          { LIST chunk - read length of the list }
          idLIST : begin
                     Readed := FMemStream.Read(ChunkLen,4);
                     if Readed <> 4 then
                       Raise Exception.Create('');
                   end;
          { INAM chunk - read length of the chunk and the Title string }
          idINAM : begin
                     Readed := FMemStream.Read(ChunkLen,4);
                     if Readed <> 4 then
                       Raise Exception.Create('');
                     { Read Title string }
                     Buffer := StrAlloc(ChunkLen+1);
                     try
                       FillChar(Buffer^,ChunkLen+1,0);
                       Readed := FMemStream.Read(Buffer^,ChunkLen);
                       FTitle := StrPas(Buffer);
                     finally
                       StrDispose(Buffer);
                     end;
                     if Readed <> ChunkLen then
                       Raise Exception.Create('');
                     { Read #0 characters if the chunk length is greater than the string }
                     if FMemStream.Position < FMemStream.Size then
                       begin
                         FMemStream.Read(B,1);
                         while (B = 0) and (FMemStream.Position < FMemStream.Size) do
                           FMemStream.Read(B,1);
                         FMemStream.Seek(-1,soFromCurrent);
                       end;
                   end;
          { IART chunk - read length of the chunk and the Author string }
          idIART : begin
                     Readed := FMemStream.Read(ChunkLen,4);
                     if Readed <> 4 then
                       Raise Exception.Create('');
                     { Read Author string }
                     Buffer := StrAlloc(ChunkLen+1);
                     try
                       FillChar(Buffer^,ChunkLen+1,0);
                       Readed := FMemStream.Read(Buffer^,ChunkLen);
                       FAuthor := StrPas(Buffer);
                     finally
                       StrDispose(Buffer);
                     end;
                     if Readed <> ChunkLen then
                       Raise Exception.Create('');
                     { Read #0 characters if the chunk length is greater than the string }
                     if FMemStream.Position < FMemStream.Size then
                       begin
                         FMemStream.Read(B,1);
                         while (B = 0) and (FMemStream.Position < FMemStream.Size) do
                           FMemStream.Read(B,1);
                         FMemStream.Seek(-1,soFromCurrent);
                       end;
                   end;
          { ICON chunk - read length of the chunk and all of the icons }
          idICON : begin
                     Readed := FMemStream.Read(ChunkLen,4);
                     if Readed <> 4 then
                       Raise Exception.Create('');
                     { Correct image type identifier from cursor to icon }
                     CMemStream := TMemoryStream.Create;
                     try
                       CMemStream.SetSize(ChunkLen);
                       CMemStream.CopyFrom(FMemStream,ChunkLen);
                       CMemStream.Seek(2,soFromBeginning);
                       CMemStream.Write(ByteOne,1);
                       CMemStream.Seek(0,soFromBeginning);
                       { Read icon }
                       Icon := TIcon.Create;
                       Icon.LoadFromStream(CMemStream);
                       FIcons.Add(Icon);
                     finally
                       CMemStream.Free;
                     end;
                   end;
          { ANIH chunk - read length of the chunk and the ANIHeader structure }
          idANIH : begin
                     Readed := FMemStream.Read(ChunkLen,4);
                     if (Readed <> 4) or (ChunkLen <> 36) then
                       Raise Exception.Create('');
                     { Read ANI header }
                     Readed := FMemStream.Read(ANIHeader,ChunkLen);
                     if Readed <> ChunkLen then
                       Raise Exception.Create('');
                     FSteps := ANIHeader.Steps;
                     FJifRate := ANIHeader.JifRate;
                   end;
          { RATE chunk - read length of the chunk and the rate table }
          idRATE : begin
                     Readed := FMemStream.Read(ChunkLen,4);
                     if Readed <> 4 then
                       Raise Exception.Create('');
                     { Read rates }
                     for I := 1 to ChunkLen div 4 do
                       begin
                         Readed := FMemStream.Read(L,4);
                         if Readed <> 4 then
                           Raise Exception.Create('');
                         FRates.Add(Pointer(L));
                       end;
                   end;
          { SEQ chunk - read length of the chunk and the sequence table }
          idSEQ  : begin
                     Readed := FMemStream.Read(ChunkLen,4);
                     if Readed <> 4 then
                       Raise Exception.Create('');
                     { Read seqences }
                     for I := 1 to ChunkLen div 4 do
                       begin
                         Readed := FMemStream.Read(L,4);
                         if Readed <> 4 then
                           Raise Exception.Create('');
                         FSeq.Add(Pointer(L));
                       end;
                   end;
          { Unrecognized chunk - error }
          else
            Raise Exception.Create('');
        end;
      end;
    { Create internal sequence table if not present }
    if FSeq.Count = 0 then
      for I := 0 to ANIHeader.Steps-1 do
        FSeq.Add(Pointer(I));
    { Create internal rate table if not present }
    if FRates.Count = 0 then
      for I := 0 to ANIHeader.Steps-1 do
        FRates.Add(Pointer(ANIHeader.JifRate));
  except
    on exception do
      begin
        MessageDlg('Error reading cursor file',mtError,[mbOk],0);
        raise;
      end;
  end;
end;

{ Called by the animation - draws the current icon }
procedure TANICursor.DrawCurIcon;
var
  Canvas : TCanvas;
begin
  Canvas := TCanvas.Create;
  try
    Canvas.Handle := GetDC(FControl.Handle);
    DrawIcon(Canvas,FX,FY,FCurStep,FWidth,FHeight,FBrush);
  finally
    ReleaseDC(FControl.Handle,Canvas.Handle);
    Canvas.Free;
  end;
end;

{ Called by the timer - continues the animation }
procedure TANICursor.TimerEvent(Sender: TObject);
begin
  if FCurStep < FSteps-1
  then
    Inc(FCurStep)
  else
    FCurStep := 0;
  DrawCurIcon;
  FTimer.Interval := Speed * LongInt(FRates[FCurStep]);
end;

{ Object creation }
constructor TANICursor.Create;
begin
  inherited Create;
  FMemStream := TMemoryStream.Create;
  FIcons := TList.Create;
  FRates := TList.Create;
  FSeq := TList.Create;
  FTimer := TTimer.Create(nil);
  FTimer.Enabled := False;
  FTimer.OnTimer := TimerEvent;
  FCurStep := 0;
end;

{ Object destoying }
destructor TANICursor.Destroy;
var
  I : Integer;
begin
  FTimer.Enabled := False;
  if FAnimation then
    FControl.Invalidate;
  for I := 0 to FIcons.Count-1 do
    TIcon(FIcons[I]).Free;
  FTimer.Free;
  FIcons.Free;
  FSeq.Free;
  FRates.Free;
  FMemStream.Free;
  inherited Destroy;
end;

{ Starts animation }
procedure TANICursor.Animate(Control : TWinControl; X,Y : Integer; Width,Height : Integer; Brush : TBrush; Start : Boolean);
begin
  if Control <> nil then
    begin
      { Hide previous animation }
      HideAnimation;
      FControl := Control;
      FX := X;
      FY := Y;
      if Width < 0
      then
        FWidth := TIcon(FIcons[0]).Width
      else
        FWidth := Width;
      if Height < 0
      then
        FHeight := TIcon(FIcons[0]).Height
      else
        FHeight := Height;
      FBrush := Brush;
      FCurStep := 0;
      if Start then
        ShowAnimation;
    end;
end;

{ Draws the Stepth icon from the animated cursor on Canvas at the X,Y position }
procedure TANICursor.DrawIcon(Canvas : TCanvas; X,Y : Integer; Step : Integer; Width,Height : Integer; Brush : TBrush);
begin
  { Correct Step }
  if Step < 0
  then
    Step := 0
  else
    if Step > FSteps-1 then
      Step := FSteps-1;
  { Correct Width }
  if Width < 0 then
    Width := TIcon(FIcons[LongInt(FSeq[Step])]).Width;
  { Correct Height }
  if Height < 0 then
    Height := TIcon(FIcons[LongInt(FSeq[Step])]).Height;
  { Draw the Stepth icon }
  if Assigned(Brush)
  then
    DrawIconEx(Canvas.Handle,X,Y,TIcon(FIcons[LongInt(FSeq[Step])]).Handle,Width,Height,0,Brush.Handle,DI_NORMAL)
  else
    DrawIconEx(Canvas.Handle,X,Y,TIcon(FIcons[LongInt(FSeq[Step])]).Handle,Width,Height,0,Canvas.Brush.Handle,DI_NORMAL);
end;

{ Shows the animation }
procedure TANICursor.ShowAnimation;
begin
  Animation := True;
end;

{ Hides the animation }
procedure TANICursor.HideAnimation;
begin
  Animation := False;
end;

{--- TANICursorList methods ---}

{ Returns the title string of the Indexth animated cursor }
function TANICursorList.GetTitle(Index : Integer) : String;
begin
  Result := TANICursor(FCursors[Index]).Title;
end;

{ Returns the author string of the Indexth animated cursor }
function TANICursorList.GetAuthor(Index : Integer) : String;
begin
  Result := TANICursor(FCursors[Index]).Author;
end;

{ Returns the number of animated cursor }
function TANICursorList.GetCount : Integer;
begin
  Result := FCursors.Count;
end;

{ Returns the step count of the Indexth animated cursor }
function TANICursorList.GetStepCount(Index: Integer) : Integer;
begin
  Result := TANICursor(FCursors[Index]).StepCount;
end;

{ Returns the Stepth icon from the Indexth animated cursor }
function TANICursorList.GetIcon(Index,Step : Integer) : TIcon;
begin
  Result := TANICursor(FCursors[Index]).Icon[Step];
end;

{ Returns the value of the Animation property of the Indexth animated cursor }
function TANICursorList.GetAnimation(Index : Integer) : Boolean;
begin
  Result := TANICursor(FCursors[Index]).Animation;
end;

{ Shows or hides the animation of the Indexth animated cursor }
procedure TANICursorList.SetAnimation(Index : Integer; AAnimation : Boolean);
begin
  TANICursor(FCursors[Index]).Animation := AAnimation;
end;

{ Called automatically to define unpublished but stored properties }
procedure TANICursorList.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineProperty('Cursors',ReadData,WriteData,FCursors.Count > 0);
end;

{ Called automatically to read in unpublished properties }
procedure TANICursorList.ReadData(Reader: TReader);
var
  ANICursor : TANICursor;
  S         : LongInt;
begin
  with Reader do
    begin
      { Start of the list }
      ReadListBegin;
      { While list items present }
      while not EndOfList do
        begin
          { Create a new ANICursor object }
          ANICursor := TANICursor.Create;
          { Read in the size }
          S := ReadInteger;
          with ANICursor do
            begin
              { Setup the memory stream length }
              FMemStream.SetSize(S);
              { Read in the memory image of the animated cursor }
              Read(FMemStream.Memory^,FMemStream.Size);
              { Setup internal fields }
              SetFields;
            end;
          FCursors.Add(ANICursor);
        end;
      { End of the list }
      ReadListEnd;
    end;
end;

{ Called automatically to write out unpublished properties }
procedure TANICursorList.WriteData(Writer: TWriter);
var
  I : Integer;
begin
  with Writer do
    begin
      { Start of the list }
      WriteListBegin;
      { Write out all animated cursors }
      for I := 0 to FCursors.Count-1 do
        with TANICursor(FCursors[I]) do
          begin
            { Write out the length of the memory stream }
            WriteInteger(FMemStream.Size);
            { Write out the memory image of the animated cursor }
            Write(FMemStream.Memory^,FMemStream.Size);
          end;
      { End of the list }
      WriteListEnd;
    end;
end;

{ Called automatically when the component is loaded }
procedure TANICursorList.Loaded;
begin
  inherited Loaded;
  { Register all animated cursors }
  if not (csDesigning in ComponentState) then
    RegisterCursors;
end;

{ Object creation }
constructor TANICursorList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  { Create a list to store animated cursor objects }
  FCursors := TList.Create;
end;

{ Object destroying }
destructor TANICursorList.Destroy;
var
  I : Integer;
begin
  { Destroy animated cursor objects }
  for I := 0 to FCursors.Count-1 do
    TANICursor(FCursors[I]).Free;
  { Destroy the list }
  FCursors.Free;
  inherited Destroy;
end;

{ Adds new animated cursor to the list from the file FileName and returns the
  index of the newly added cursor }
function TANICursorList.Add(const FileName: TFileName): Integer;
begin
  Result := FCursors.Count;
  Insert(FCursors.Count,FileName);
end;

{ Like Add, but don't returns the index of the newly added cursor }
procedure TANICursorList.Append(const FileName: TFileName);
begin
  Add(FileName);
end;

{ Deletes all animated cursors }
procedure TANICursorList.Clear;
var
  I : Integer;
begin
  for I := 0 to FCursors.Count-1 do
    TANICursor(FCursors[I]).Free;
  FModified := True;
  if not FInUpdate and Assigned(FOnChange) and not (csDesigning in ComponentState) then
    FOnChange(Self);
end;

{ Deletes the Indexth animated cursor }
procedure TANICursorList.Delete(Index: Integer);
begin
  TANICursor(FCursors[Index]).Free;
  FCursors.Delete(Index);
  FModified := True;
  if not FInUpdate and not (csDesigning in ComponentState) then
    begin
      RegisterCursors;
      if Assigned(FOnChange) then
        FOnChange(Self);
    end;
end;

{ Exchanges two cursors in the list }
procedure TANICursorList.Exchange(Index1, Index2: Integer);
begin
  FCursors.Exchange(Index1,Index2);
  FModified := True;
  if not FInUpdate and not (csDesigning in ComponentState) then
    begin
      RegisterCursors;
      if Assigned(FOnChange) then
        FOnChange(Self);
    end;
end;

{ Finds an animated cursor by the Title string, sets Index to the index of the
  animated cursor and returns True, if the animated cursor is in the list }
function TANICursorList.Find(const Title: string; var Index: Integer): Boolean;
begin
  Result := False;
  Index := 0;
  while not Result and (Index < FCursors.Count) do
    begin
      Result := TANICursor(FCursors[Index]).Title = Title;
      if not Result then
        Inc(Index);
    end;
end;

{ Returns the index of the animated cursor identified by Title }
function TANICursorList.IndexOf(const Title: string): Integer;
var
  I : Integer;
begin
  if Find(Title,I)
  then
    Result := I
  else
    Result := -1;
end;

{ Inserts FileName animated cursor at the Indexth position }
procedure TANICursorList.Insert(Index: Integer; const FileName: TFileName);
var
  ANICursor  : TANICursor;
  FileStream : TFileStream;
begin
  try
    FileStream := TFileStream.Create(FileName,fmOpenRead);
    try
      ANICursor := TANICursor.Create;
      ANICursor.FMemStream.CopyFrom(FileStream,0);
      try
        ANICursor.SetFields;
        FCursors.Insert(Index,ANICursor);
        FModified := True;
        if not FInUpdate and not (csDesigning in ComponentState) then
          begin
            RegisterCursors;
            if Assigned(FOnChange) then
              FOnChange(Self);
          end;
      except
        ANICursor.Free;
      end;
    finally
      FileStream.Free;
    end;
  except
    on exception do
      MessageDlg('File ' + ExtractFileName(FileName) + ' not found',mtError,[mbOk],0);
  end;
end;

{ Moves CurIndexth animated cursor to the NewIndex position in the list }
procedure TANICursorList.Move(CurIndex, NewIndex: Integer);
begin
  FCursors.Move(CurIndex,NewIndex);
  FModified := True;
  if not FInUpdate and not (csDesigning in ComponentState) then
    begin
      RegisterCursors;
      if Assigned(FOnChange) then
        FOnChange(Self);
    end;
end;

{ Registers all cursors from the list }
procedure TANICursorList.RegisterCursors;
var
  FileStream   : TFileStream;
  TempPathName : Array[0..255] of Char;
  TempFileName : Array[0..255] of Char;
  I            : Integer;
  S            : String;
begin
  Screen.Cursor := crHourGlass;
  try
    for I := 0 to FCursors.Count-1 do
      begin
        { Save the animated cursor to a temporary file }
        GetTempPath(255,@TempPathName);
        GetTempFileName(@TempPathName,'ANI',0,@TempFileName);
        S := StrPas(@TempFileName);
        FileStream := TFileStream.Create(S,fmOpenWrite);
        try
          FileStream.CopyFrom(TANICursor(FCursors[I]).FMemStream,0);
        finally
          FileStream.Free;
        end;
        { Load the cursor from the temporary file }
        Screen.Cursors[I+1] := LoadCursorFromFile(@TempFileName);
        { Delete the temporary file }
        DeleteFile(StrPas(@TempFileName));
      end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

{ Sets Indexth animated cursor as screen cursor or if Index is less than zero,
  restores the default cursor }
procedure TANICursorList.SetCursor(Index : Integer);
begin
  if Index < 0
  then
    Screen.Cursor := crDefault
  else
    Screen.Cursor := Index + 1;
end;

{ Starts or stops the animation of the Indexth animated cursor }
procedure TANICursorList.Animate(Index : Integer; Control : TWinControl; X,Y : Integer; Width,Height : Integer; Brush : TBrush; Start : Boolean);
begin
  TANICursor(FCursors[Index]).Animate(Control,X,Y,Width,Height,Brush,Start);
end;

{ Draws the Stepth icon of the Indexth animated cursor }
procedure TANICursorList.DrawIcon(Index,Step : Integer; Canvas : TCanvas; X,Y : Integer; Width,Height : Integer; Brush : TBrush);
begin
  TANICursor(FCursors[Index]).DrawIcon(Canvas,X,Y,Step,Width,Height,Brush);
end;

{ Shows the animation of the Indexth animated cursor }
procedure TANICursorList.ShowAnimation(Index : Integer);
begin
  TANICursor(FCursors[Index]).ShowAnimation;
end;

{ Hides the animation of the Indexth animated cursor }
procedure TANICursorList.HideAnimation(Index : Integer);
begin
  TANICursor(FCursors[Index]).HideAnimation;
end;

{ Shows the animation of all animated cursors }
procedure TANICursorList.ShowAnimations;
var
  I : Integer;
begin
  for I := 0 to FCursors.Count-1 do
    TANICursor(FCursors[I]).ShowAnimation;
end;

{ Hides the animation of all animated cursors }
procedure TANICursorList.HideAnimations;
var
  I : Integer;
begin
  for I := 0 to FCursors.Count-1 do
    TANICursor(FCursors[I]).HideAnimation;
end;

{ Starts update }
procedure TANICursorList.BeginUpdate;
begin
  FModified := False;
  FInUpdate := True;
end;

{ Stops update }
procedure TANICursorList.EndUpdate;
begin
  if FModified and FInUpdate then
    begin
      RegisterCursors;
      If Assigned(FOnChange) then
        FOnChange(Self);
    end;
  FInUpdate := False;
end;

{--- TANICursorListEditor ---}

procedure TANICursorListEditor.Edit;
var
  D : TANICursorListEditorDlg;
begin
  D := TANICursorListEditorDlg.Create(nil);
  try
    D.Component := Component;
    D.Designer := Designer;
    D.ShowModal;
  finally
    D.Free;
  end;
end;

procedure TANICursorListEditor.ExecuteVerb(Index: Integer);
begin
  Edit;
end;

function TANICursorListEditor.GetVerb(Index: Integer): string;
begin
  Result := 'Cursorlist editor...';
end;

function TANICursorListEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

{--- Registering ---}

procedure Register;
begin
  RegisterComponents('Win32',[TANICursorList]);
  RegisterComponentEditor(TANICursorList,TANICursorListEditor);
end;

end.
